$debug
      SUBROUTINE QC25C(F,A,B,C,RESULT,ABSERR,KRUL,NEVAL)
C
C......................................................................
C
C 1.     QC25C
C        INTEGRATION RULES FOR THE COMPUTATION OF CAUCHY
C        PRINCIPAL VALUE INTEGRALS
C
C 2.     PURPOSE
C           TO COMPUTE  I = INTEGRAL OF F*W OVER (A,B) WITH ERROR
C           ESTIMATE, WHERE W(X)=1/(X-C)
C
C 3.     CALLING SEQUENCE
C           CALL QC25C(F,A,B,C,RESULT,ABSERR,KRUL,NEVAL)
C
C        PARAMETERS
C           F      -REAL
C                   FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                   F(X). THE ACTUAL NAME FOR F NEEDS TO BE DECLARED
C                   E X T E R N A L  IN THE DRIVER PROGRAM.
C
C           A      -REAL
C                   LEFT END POINT OF THE INTEGRATION INTERVAL
C
C           B      -REAL
C                   RIGHT END POINT OF THE INTEGRATION INTERVAL,
C                   B.GT.A
C
C           C      -REAL
C                   PARAMETER IN THE WEIGHT FUNCTION
C
C           RESULT -REAL
C                   APPROXIMATION TO THE INTEGRAL
C                   RESULT IS COMPUTED BY USING A GENERALIZED
C                   CLENSHAW-CURTIS METHOD IF C LIES WITHIN TEN PERCENT
C                   OF THE INTEGRATION INTERVAL. IN THE OTHER CASE THE
C                   15-POINT KRONROD RULE OBTAINED BY OPTIMAL ADDITION
C                   OF ABSCISSAE TO THE 7-POINT GAUSS RULE, IS APPLIED.
C
C           ABSERR -REAL
C                   ESTIMATE OF THE MODULUS OF THE ABSOLUTE ERROR,
C                   WHICH SHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C           KRUL   -INTEGER
C                   KEY WHICH IS DECREASED BY 1 IF THE 15-POINT
C                   GAUSS-KRONROD SCHEME HAS BEEN USED
C
C           NEVAL  -INTEGER
C                   NUMBER OF INTEGRAND EVALUATIONS
C
C 4.     SUBROUTINES OR FUNCTIONS NEEDED
C              - QCHEB
C              - QK15W
C              - F (USER-PROVIDED FUNCTION)
C              - QWGTC
C              - FORTRAN ABS, ALOG, AMAX1, AMIN1
C
C.......................................................................
C
      REAL*8 A,ABSERR,AK22,AMOM0,AMOM1,AMOM2,B,C,CC,CENTR,CHEB12,
     *  CHEB24,F,
     *  FVAL,HLGTH,P2,P3,P4,QWGTC,RESABS,RESASC,RESULT,RES12,RES24,U,X
      INTEGER I,ISYM,K,KP,KRUL,NEVAL
C
      DIMENSION CHEB12(13),CHEB24(25),FVAL(25),X(11)
C
      EXTERNAL F,QWGTC
C
C           THE VECTOR X CONTAINS THE VALUES COS(K*PI/24),
C           K = 1, ..., 11, TO BE USED FOR THE CHEBYSHEV SERIES
C           EXPANSION OF F
C
      DATA X(1),X(2),X(3),X(4),X(5),X(6),X(7),X(8),X(9),X(10),X(11)/
     *     9.914448613738104D-01,        9.659258262890683D-01,
     *     9.238795325112868D-01,        8.660254037844386D-01,
     *     7.933533402912352D-01,        7.071067811865475D-01,
     *     6.087614290087206D-01,        5.000000000000000D-01,
     *     3.826834323650898D-01,        2.588190451025208D-01,
     *     1.305261922200516D-01/
C
C           LIST OF MAJOR VARIABLES
C           -----------------------
C           FVAL   -VALUE OF THE FUNCTION F AT THE POINTS
C                   COS(K*PI/24),  K=0, ..., 24
C           CHEB12 -CHEBYSHEV SERIES EXPANSION COEFFICIENTS, FOR THE
C                   FUNCTION F, OF DEGREE 12
C           CHEB24 -CHEBYSHEV SERIES EXPANSION COEFFICIENTS, FOR THE
C                   FUNCTION F, OF DEGRE 24
C           RES12  -APPROXIMATION TO THE INTEGRAL CORRESPONDING TO THE
C                   USE OF CHEB12
C           RES24  -APPROXIMATION TO THE INTEGRAL CORRESPONDING TO THE
C                   USE OF CHEB24
C           QWGTC  -EXTERNAL FUNCTION SUBPROGRAM DEFINING THE WEIGHT
C                   FUNCTION
C           HLGTH  -HALF-LENGTH OF THE INTERVAL
C           CENTR  -MID POINT OF THE INTERVAL
C
C
C           CHECK THE POSITION OF C.
C
C***FIRST EXECUTABLE STATEMENT
      CC = (2.0D+00*C-B-A)/(B-A)
      IF(DABS(CC).LT.1.1D+00) GO TO 10
C
C           APPLY THE 15-POINT GAUSS-KRONROD SCHEME
C
      KRUL = KRUL-1
      CALL QK15W(F,QWGTC,C,P2,P3,P4,KP,A,B,RESULT,ABSERR,RESABS,RESASC)
      NEVAL = 15
      IF(RESASC.EQ.ABSERR) KRUL=KRUL+1
      GO TO 50
C
C           USE THE GENERALIZED CLENSHAW-CURTIS METHOD.
C
   10 HLGTH = 5.0D-01*(B-A)
      CENTR = 5.0D-01*(B+A)
      NEVAL = 25
      FVAL(1) = 5.0D-01*F(HLGTH+CENTR)
      FVAL(13) = F(CENTR)
      FVAL(25) = 5.0D-01*F(CENTR-HLGTH)
      DO 20 I=2,12
        U = HLGTH*X(I-1)
        ISYM = 26-I
        FVAL(I) = F(U+CENTR)
        FVAL(ISYM) = F(CENTR-U)
   20 CONTINUE
C
C           COMPUTE THE CHEBYSHEV SERIES EXPANSION.
C
      CALL QCHEB(X,FVAL,CHEB12,CHEB24)
C
C           THE MODIFIED CHEBYSHEV MOMENTS ARE COMPUTED BY FORWARD
C           RECURSION, USING AMOM0 AND AMOM1 AS STARTING VALUES.
C
      AMOM0 = DLOG(DABS((1.0D+00-CC)/(1.0D+00+CC)))
      AMOM1 = 2.0D+00+CC*AMOM0
      RES12 = CHEB12(1)*AMOM0+CHEB12(2)*AMOM1
      RES24 = CHEB24(1)*AMOM0+CHEB24(2)*AMOM1
      DO 30 K=3,13
        AMOM2 = 2.0D+00*CC*AMOM1-AMOM0
        AK22 = (K-2)*(K-2)
        IF((K/2)*2.EQ.K) AMOM2=AMOM2-4.0D+00/(AK22-1.0D+00)
        RES12 = RES12+CHEB12(K)*AMOM2
        RES24 = RES24+CHEB24(K)*AMOM2
        AMOM0 = AMOM1
        AMOM1 = AMOM2
   30 CONTINUE
      DO 40 K=14,25
        AMOM2 = 2.0D+00*CC*AMOM1-AMOM0
        AK22 = (K-2)*(K-2)
        IF((K/2)*2.EQ.K) AMOM2=AMOM2-4.0D+00/(AK22-1.0D+00)
        RES24 = RES24+CHEB24(K)*AMOM2
        AMOM0 = AMOM1
        AMOM1 = AMOM2
   40 CONTINUE
      RESULT = RES24
      ABSERR = DABS(RES24-RES12)
   50 RETURN
      END
