      SUBROUTINE QC25O(F,A,B,OMEGA,INTEGR,NRMOM,MAXP1,KSAVE,RESULT,
     *  ABSERR,NEVAL,RESABS,RESASC,MOMCOM,CHEBMO)
C
C......................................................................
C
C 1.     QC25O
C        INTEGRATION RULES FOR FUNCTIONS WITH COS OR SIN FACTOR
C           STANDARD FORTRAN SUBROUTINE
C
C 2.     PURPOSE
C           TO COMPUTE THE INTEGRAL
C              I = INTEGRAL OF F(X)*W(X) OVER (A,B)
C                  WHERE W(X) = COS(OMEGA*X)
C                     OR W(X) = SIN(OMEGA*X)
C           AND TO COMPUTE J = INTEGRAL OF ABS(F) OVER (A,B).
C           FOR SMALL VALUES OF OMEGA OR SMALLINTERVALS (A,B) THE
C           15-POINT GAUSS-KRONROD RULE IS USED.IN ALL OTHER CASES A
C           GENERALIZED CLENSHAW-CURTIS METHOD IS USED, I.E. A
C           TRUNCATED CHEBYSHEV EXPANSION OF THE FUNCTION F IS COMPUTED
C           ON (A,B), SO THAT THE INTEGRAND CAN BE WRITTEN AS A SUM OF
C           TERMS OF THE FORM W(X)T(K,X), WHERE T(K,X) IS THE CHEBYSHEV
C           POLYNOMIAL OF DEGREE K. THE CHEBYSHEV MOMENTS ARE COMPUTED
C           WITH USE OF A LINEAR RECURRENCE RELATION.
C
C 3.     CALLING SEQUENCE
C           CALL QC25O(F,A,B,OMEGA,INTEGR,NRMOM,MAXP1,KSAVE,RESULT,
C                      ABSERR,NEVAL,RESABS,RESASC,MOMCOM,CHEBMO)
C
C        PARAMETERS
C         ON ENTRY
C           F      -REAL
C                   FUNCTION SUBPROGRAM DEFINING THE INTEGRAND
C                   FUNCTION F(X). THE ACTUAL NAME FOR F NEEDS TO
C                   BE DECLARED E X T E R N A L IN THE CALLING PROGRAM.
C
C           A      -REAL
C                   LOWER LIMIT OF INTEGRATION
C
C           B      -REAL
C                   UPPER LIMIT OF INTEGRATION
C
C           OMEGA  -REAL
C                   PARAMETER IN THE WEIGHT FUNCTION
C
C           INTEGR -INTEGER
C                   INDICATES WHICH WEIGHT FUNCTION IS TO BE USED
C                      INTEGR = 1   W(X) = COS(OMEGA*X)
C                      INTEGR = 2   W(X) = SIN(OMEGA*X)
C
C           NRMOM  -INTEGER
C                   THE LENGTH OF INTERVAL (A,B) IS EQUAL TO THE LENGTH
C                   OF THE ORIGINAL INTEGRATION INTERVAL DIVIDED BY
C                   2**NRMOM (WE SUPPOSE THAT THE ROUTINE IS USED IN AN
C                   ADAPTIVE INTEGRATION PROCESS, OTHERWISE SET
C                   NRMOM = 0). NRMOM MUST BE ZERO AT THE FIRST CALL.
C
C           MAXP1  -INTEGER
C                   GIVES AN UPPER BOUND ON THE NUMBER OF CHEBYSHEV
C                   MOMENTS WHICH CAN BE STORED, I.E. FOR THE INTERVALS
C                   OF LENGTHS ABS(BB-AA)*2**(-L), L=0,1,2,...,
C                   MAXP1-2.
C
C           KSAVE  -INTEGER
C                   KEY WHICH IS ONE WHEN THE MOMENTS FOR THE
C                   CURRENT INTERVAL HAVE BEEN COMPUTED
C
C         ON RETURN
C           RESULT -REAL
C                   APPROXIMATION TO THE INTEGRAL I
C
C           ABSERR -REAL
C                   ESTIMATE OF THE MODULUS OF THE ABSOLUTE
C                   ERROR, WHICH SHOULD EQUAL OR EXCEED ABS(I-RESULT)
C
C           NEVAL  -INTEGER
C                   NUMBER OF INTEGRAND EVALUATIONS
C
C           RESABS -REAL
C                   APPROXIMATION TO THE INTEGRAL J
C
C           RESASC -REAL
C                   APPROXIMATION TO THE INTEGRAL OF ABS(F-I/(B-A))
C
C         ON ENTRY AND RETURN
C           MOMCOM -INTEGER
C                   FOR EACH INTERVAL LENGTH WE NEED TO COMPUTE
C                   THE CHEBYSHEV MOMENTS. MOMCOM COUNTS THE NUMBER
C                   OF INTERVALS FOR WHICH THESE MOMENTS HAVE ALREADY
C                   BEEN COMPUTED. IF NRMOM.LT.MOMCOM OR KSAVE = 1,
C                   THE CHEBYSHEV MOMENTS FOR THE INTERVAL (A,B)
C                   HAVE ALREADY BEEN COMPUTED AND STORED, OTHERWISE
C                   WE COMPUTE THEM AND WE INCREASE MOMCOM.
C
C           CHEBMO -REAL
C                   ARRAY OF DIMENSION AT LEAST (MAXP1,25) CONTAINING
C                   THE MODIFIED CHEBYSHEV MOMENTS FOR THE FIRST MOMCOM
C                   INTERVAL LENGTHS
C
C 4.     SUBROUTINES OR FUNCTIONS NEEDED
C              - QK15W
C              - QCHEB
C              - QMACO
C              - F(USER-PROVIDED FUNCTION)
C              - QWGTO
C              - FORTRAN ABS, COS, ALOG, AMAX1, AMIN1, SIN, INT, MIN0
C
C......................................................................
C
      REAL*8 A,ABSERR,AC,AN,AN2,AS,ASAP,ASS,B,CENTR,CHEBMO,
     *   CHEB12,CHEB24,CONC,CONS,COSPAR,D,DABS,DCOS,DSIN,QWGTO,
     *   D1,D2,D3,EPMACH,ESTC,ESTS,F,FVAL,HLGTH,OFLOW,OMEGA,PARINT,PAR2,
     *   PAR22,P2,P3,P4,RESABS,RESASC,RESC12,RESC24,RESS12,RESS24,
     *   RESULT,SINPAR,UFLOW,V,X
      INTEGER I,INTEGR,ISYM,J,K,KSAVE,M,MOMCOM,NEVAL,
     *   NOEQU,NOEQ1,NRMOM
C
      DIMENSION CHEBMO(MAXP1,25),CHEB12(13),CHEB24(25),D(43),D1(43),
     *   D2(43),D3(43),FVAL(25),V(43),X(11)
C
      EXTERNAL F,QWGTO
C
C           THE DATA VALUE OF MAXP1 GIVES AN UPPER BOUND
C           ON THE NUMBER OF CHEBYSHEV MOMENTS WHICH CAN BE
C           COMPUTED, I.E. FOR THE INTERVAL (BB-AA), ...,
C           (BB-AA)/2**(MAXP1-2).
C           SHOULDTHIS NUMBER BE ALTERED, THE FIRST DIMENSION OF
C           CHEBMO NEEDS TO BE ADAPTED.
C
C
C           THE VECTOR X CONTAINS THE VALUES COS(K*PI/24)
C           K = 1, ...,11, TO BE USED FOR THE CHEBYSHEV EXPANSION OF F
C
      DATA X(1),X(2),X(3),X(4),X(5),X(6),X(7),X(8),X(9),
     *   X(10),X(11)/
     *      9.914448613738104D-01,      9.659258262890683D-01,
     *      9.238795325112868D-01,      8.660254037844386D-01,
     *      7.933533402912352D-01,      7.071067811865475D-01,
     *      6.087614290087206D-01,      5.000000000000000D-01,
     *      3.826834323650898D-01,      2.588190451025208D-01,
     *      1.305261922200516D-01/
C
C
C           LIST OF MAJOR VARIABLES
C           -----------------------
C           CENTR  -MID POINT OF THE INTEGRATION INTERVAL
C           HLGTH  -HALF LENGTH OF THE INTEGRATION INTERVAL
C           FVAL   -VALUE OF THE FUNCTION F AT THE POINTS
C                   (B-A)*0.5*COS(K*PI/12)+(B+A)*0.5
C                   K = 0, ...,24
C           CHEB12 -COEFFICIENTS OF THE CHEBYSHEV SERIES EXPANSION
C                   OF DEGREE 12, FOR THE FUNCTION F, IN THE
C                   INTERVAL (A,B)
C           CHEB24 -COEFFICIENTS OF THE CHEBYSHEV SERIES EXPANSION
C                   OF DEGREE 24, FOR THE FUNCTION F, IN THE
C                   INTERVAL (A,B)
C           RESC12 -APPROXIMATION TO THE INTEGRAL OF
C                   COS(0.5*(B-A)*OMEGA*X)*F(0.5*(B-A)*X+0.5*(B+A))
C                   OVER (-1,+1), USING THE CHEBYSHEV SERIES
C                   EXPANSION OF DEGREE 12
C           RESC24 -APPROXIMATION TO THE SAME INTEGRAL, USING THE
C                   CHEBYSHEV SERIES EXPANSION OF DEGREE 24
C           RESS12  -THE ANALOGUE OF RESC12 FOR THE SINE
C           RESS24  -THE ANALOGUE OF RESS24 FOR THE SINE
C
C
C           MACHINE DEPENDENT CONSTANT
C           --------------------------
C
C           OFLOW IS THE LARGEST POSITIVE MAGNITUDE.
C
      CALL QMACO(EPMACH,UFLOW,OFLOW)
C
      CENTR = 5.0D-01*(B+A)
      HLGTH = 5.0D-01*(B-A)
      PARINT = OMEGA*HLGTH
C
C           COMPUTE THE INTEGRAL USING THE 15-POINT GAUSS-KRONROD
C           FORMULA IF THE VALUE OF THE PARAMETER IN THE INTEGRAND
C           IS SMALL OR IF THE LENGTH OF THE INTEGRATION INTERVAL
C           IS LESS THAN (BB-AA)/2**(MAXP1-2), WHERE (AA,BB) IS THE
C           ORIGINAL INTEGRATION INTERVAL
C
      IF(DABS(PARINT).GT.2.0D+00) GO TO 10
      CALL QK15W(F,QWGTO,OMEGA,P2,P3,P4,INTEGR,A,B,RESULT,
     *   ABSERR,RESABS,RESASC)
      NEVAL = 15
      GO TO 190
C
C           COMPUTE THE INTEGRAL USING THE GENERALIZED CLENSHAW-
C           CURTIS METHOD
C
   10 CONC = HLGTH*DCOS(CENTR*OMEGA)
      CONS = HLGTH*DSIN(CENTR*OMEGA)
      RESASC = OFLOW
      NEVAL = 25
C
C           CHECK WHETHER THE CHEBYSHEV MOMENTS FOR THIS INTERVAL
C           HAVE ALREADY BEEN COMPUTED
C
      IF(NRMOM.LT.MOMCOM.OR.KSAVE.EQ.1) GO TO 140
C
C           COMPUTE A NEW SET OF CHEBYSHEV MOMENTS
C
      M = MOMCOM+1
      PAR2 = PARINT*PARINT
      PAR22 = PAR2+2.0D+00
      SINPAR = DSIN(PARINT)
      COSPAR = DCOS(PARINT)
C
C           COMPUTE THE CHEBYSHEV MOMENTS WITH RESPECT TO COSINE
C
      V(1) = 2.0D+00*SINPAR/PARINT
      V(2) = (8.0D+00*COSPAR+(PAR2+PAR2-8.0D+00)*SINPAR/
     *   PARINT)/PAR2
      V(3) = (3.2D+01*(PAR2-1.2D+01)*COSPAR+(2.0D+00*
     *   ((PAR2-8.0D+01)*PAR2+1.92D+02)*SINPAR)/
     *   PARINT)/(PAR2*PAR2)
      AC = 8.0D+00*COSPAR
      AS = 2.4D+01*PARINT*SINPAR
      IF(DABS(PARINT).GT.2.4D+01) GO TO 70
C
C           COMPUTE THE CHEBYSHEV MOMENTS AS THE
C           SOLUTIONS OF A BOUNDARY VALUE PROBLEM WITH 1
C           INITIAL VALUE (V(3)) AND 1 END VALUE (COMPUTED
C           USING AN ASYMPTOTIC FORMULA)
C
      NOEQU = MIN0(40,13+INT(-DLOG(EPMACH))/3)
      NOEQ1 = NOEQU-1
      AN = 6.0D+00
      DO 20 K=1,NOEQ1
        AN2 = AN*AN
        D(K) = -2.0D+00*(AN2-4.0D+00)*(PAR22-AN2-AN2)
        D2(K) = (AN-1.0D+00)*(AN-2.0D+00)*PAR2
        D1(K) = (AN+3.0D+00)*(AN+4.0D+00)*PAR2
        V(K+3) = AS-(AN2-4.0D+00)*AC
        AN = AN+2.0D+00
   20 CONTINUE
      AN2 = AN*AN
      D(NOEQU) = -2.0D+00*(AN2-4.0D+00)*(PAR22-AN2-AN2)
      V(NOEQU+3) = AS-(AN2-4.0D+00)*AC
      V(4) = V(4)-5.6D+01*PAR2*V(3)
      ASS = PARINT*SINPAR
      ASAP = (((((2.10D+02*PAR2-1.0D+00)*COSPAR-(1.05D+02*PAR2
     *   -6.3D+01)*ASS)/AN2-(1.0D+00-1.5D+01*PAR2)*COSPAR
     *   +1.5D+01*ASS)/AN2-COSPAR+3.0D+00*ASS)/AN2-COSPAR)/AN2
      V(NOEQU+3) = V(NOEQU+3)-2.0D+00*ASAP*PAR2*(AN-1.0D+00)*
     *   (AN-2.0D+00)
C
C           SOLVE THE TRIDIAGONAL SYSTEM BY MEANS OF GAUSSIAN
C           ELIMINATION WITH PARTIAL PIVOTING
C
      DO 30 I=1,NOEQU
        D3(I) = 0.0D+00
   30 CONTINUE
      D2(NOEQU) = 0.0D+00
      DO 50 I=1,NOEQ1
        IF(DABS(D1(I)).LE.DABS(D(I))) GO TO 40
        AN = D1(I)
        D1(I) = D(I)
        D(I) = AN
        AN = D2(I)
        D2(I) = D(I+1)
        D(I+1) = AN
        D3(I) = D2(I+1)
        D2(I+1) = 0.0D+00
        AN = V(I+4)
        V(I+4) = V(I+3)
        V(I+3) = AN
   40   D(I+1) = D(I+1)-D2(I)*D1(I)/D(I)
        D2(I+1) = D2(I+1)-D3(I)*D1(I)/D(I)
        V(I+4) = V(I+4)-V(I+3)*D1(I)/D(I)
   50 CONTINUE
      V(NOEQU+3) = V(NOEQU+3)/D(NOEQU)
      V(NOEQU+2) = (V(NOEQU+2)-D2(NOEQ1)*V(NOEQU+3))/D(NOEQ1)
      DO 60 I=2,NOEQ1
        K = NOEQU-I
        V(K+3) = (V(K+3)-D3(K)*V(K+5)-D2(K)*V(K+4))/D(K)
   60 CONTINUE
      GO TO 90
C
C           COMPUTE THE CHEBYSHEV MOMENTS BY MEANS OF FORWARD
C           RECURSION
C
   70 AN = 4.0D+00
      DO 80 I=4,13
        AN2 = AN*AN
        V(I) = ((AN2-4.0D+00)*(2.0D+00*(PAR22-AN2-AN2)*V(I-1)-AC)
     *   +AS-PAR2*(AN+1.0D+00)*(AN+2.0D+00)*V(I-2))/
     *   (PAR2*(AN-1.0D+00)*(AN-2.0D+00))
        AN = AN+2.0D+00
   80 CONTINUE
   90 DO 100 J=1,13
        CHEBMO(M,2*J-1) = V(J)
  100 CONTINUE
C
C           COMPUTE THE CHEBYSHEV MOMENTS WITH RESPECT TO SINE
C
      V(1) = 2.0D+00*(SINPAR-PARINT*COSPAR)/PAR2
      V(2) = (1.8D+01-4.8D+01/PAR2)*SINPAR/PAR2
     *   +(-2.0D+00+4.8D+01/PAR2)*COSPAR/PARINT
      AC = -2.4D+01*PARINT*COSPAR
      AS = -8.0D+00*SINPAR
      CHEBMO(M,2) = V(1)
      CHEBMO(M,4) = V(2)
      IF(DABS(PARINT).GT.2.4D+01) GO TO 120
      DO 110 K=3,12
        AN = K
        CHEBMO(M,2*K) = -SINPAR/(AN*(2.0D+00*AN-2.0D+00))
     *               -2.5D-01*PARINT*(V(K+1)/AN-V(K)/(AN-1.0D+00))
  110 CONTINUE
      GO TO 140
C
C           COMPUTE THE CHEBYSHEV MOMENTS BY MEANS OF
C           FORWARD RECURSION
C
  120 AN = 3.0D+00
      DO 130 I=3,12
        AN2 = AN*AN
        V(I) = ((AN2-4.0D+00)*(2.0D+00*(PAR22-AN2-AN2)*V(I-1)+AS)
     *   +AC-PAR2*(AN+1.0D+00)*(AN+2.0D+00)*V(I-2))
     *   /(PAR2*(AN-1.0D+00)*(AN-2.0D+00))
        AN = AN+2.0D+00
        CHEBMO(M,2*I) =V(I)
  130 CONTINUE
  140 IF(NRMOM.LT.MOMCOM) M=NRMOM+1
      IF(MOMCOM.LT.(MAXP1-1).AND.NRMOM.GE.MOMCOM) MOMCOM=MOMCOM+1
C
C           COMPUTE THE COEFFICIENTS OF THE CHEBYSHEV EXPANSIONS
C           OF DEGREES 12 AND 24 OF THE FUNCTION F
C
      FVAL(1) = 5.0D-01*F(CENTR+HLGTH)
      FVAL(13) = F(CENTR)
      FVAL(25) = 5.0D-01*F(CENTR-HLGTH)
      DO 150 I=2,12
        ISYM = 26-I
        FVAL(I) = F(HLGTH*X(I-1)+CENTR)
        FVAL(ISYM) = F(CENTR-HLGTH*X(I-1))
  150 CONTINUE
      CALL QCHEB(X,FVAL,CHEB12,CHEB24)
C
C           COMPUTE THE INTEGRAL AND ERROR ESTIMATES
C
      RESC12 = CHEB12(13)*CHEBMO(M,13)
      RESS12 =0.0D+00
      ESTC = DABS(CHEB24(25)*CHEBMO(M,25))+DABS((CHEB12(13)-
     *  CHEB24(13))*CHEBMO(M,13))
      ESTS = 0.0D+00
      K = 11
      DO 160 J=1,6
        RESC12 = RESC12+CHEB12(K)*CHEBMO(M,K)
        RESS12 = RESS12+CHEB12(K+1)*CHEBMO(M,K+1)
        ESTC = ESTC+DABS((CHEB12(K)-CHEB24(K))*CHEBMO(M,K))
        ESTS = ESTS+DABS((CHEB12(K+1)-CHEB24(K+1))*CHEBMO(M,K+1))
        K = K-2
  160 CONTINUE
      RESC24 = CHEB24(25)*CHEBMO(M,25)
      RESS24 = 0.0D+00
      RESABS = DABS(CHEB24(25))
      K = 23
      DO 170 J=1,12
        RESC24 = RESC24+CHEB24(K)*CHEBMO(M,K)
        RESS24 = RESS24+CHEB24(K+1)*CHEBMO(M,K+1)
        RESABS = RESABS+DABS(CHEB24(K))+DABS(CHEB24(K+1))
        IF(J.LE.5) ESTC = ESTC+DABS(CHEB24(K)*CHEBMO(M,K))
        IF(J.LE.5) ESTS = ESTS+DABS(CHEB24(K+1)*CHEBMO(M,K+1))
        K = K-2
  170 CONTINUE
      RESABS = RESABS*DABS(HLGTH)
      IF(INTEGR.EQ.2) GO TO 180
      RESULT = CONC*RESC24-CONS*RESS24
      ABSERR = DABS(CONC*ESTC)+DABS(CONS*ESTS)
      GO TO 190
  180 RESULT = CONC*RESS24+CONS*RESC24
      ABSERR = DABS(CONC*ESTS)+DABS(CONS*ESTC)
  190 RETURN
      END




